#include "Menu.h"

Menu::Menu() : _items(MenuEntries)
{
  //Load textures
  _tile = oslLoadImageFilePNG((char*)"media/tile.png", OSL_IN_RAM, OSL_PF_8888);
  oslSetImageFrameSize(_tile, Game::BrickSize, Game::BrickSize);
  _fadeStatus = FadingIn;
  _fadeAlpha = 0xff;
  _currentItem = 0;
  //Load sounds
  _switch = oslLoadSoundFile("sfx/shortbip.wav", OSL_FMT_NONE);
  _validate = oslLoadSoundFile("sfx/validate.wav", OSL_FMT_NONE);
  //Create items
  _speedBar = new ProgressBar(1, 10, MenuOffsetX + 40, MenuOffsetY + MenuItemSpacingY * 2 + 2);
  _levelBar = new ProgressBar(1, 10, MenuOffsetX + 40, MenuOffsetY + MenuItemSpacingY * 3 + 2);
  _modeText = new Text("TEXT", MenuOffsetX + 36/*TEST*/, MenuOffsetY + MenuItemSpacingY);
  _modeText->setColor(RGBA(255, 255, 255, 0));
  _items[0] = new Text("New Game", MenuOffsetX, MenuOffsetY);
  _items[1] = new Text("Mode:", MenuOffsetX, MenuOffsetY + MenuItemSpacingY);
  _items[1]->addChild(_modeText);
  _items[2] = new Text("Speed", MenuOffsetX, MenuOffsetY + MenuItemSpacingY * 2);
  _items[2]->addChild(_speedBar);
  _items[3] = new Text("Level", MenuOffsetX, MenuOffsetY + MenuItemSpacingY * 3);
  _items[3]->addChild(_levelBar);
  _items[4] = new Text("Quit", MenuOffsetX, MenuOffsetY + MenuItemSpacingY * 4);
  _items[_currentItem]->setFocus(true);
  setMode(Classic); //setMode() calls _items-dependant functions
  showIntro();
}

Menu::~Menu()
{
  oslDeleteSound(_validate);
  oslDeleteSound(_switch);
  oslDeleteImage(_tile);
  //FIXME proper deletion
  for (std::vector<Item*>::iterator it = _items.begin(); it != _items.end(); ++it) {
    delete (*it);
  }
}

void Menu::update(OSL_CONTROLLER* pad)
{
  //Compute fades
  if (_fadeStatus == FadingIn) {
    _fadeAlpha -= 4;
    if (_fadeAlpha <= 0) {
      _fadeAlpha = 0;
      _fadeStatus = FadedIn;
    }
  }
  if (_fadeStatus == FadingOut) {
    _fadeAlpha += 4;
    if (_fadeAlpha >= 0xff) {
      _fadeAlpha = 0xff;
      _fadeStatus = FadedOut;
    }
  }
  //Test for input
  if (pad->pressed.down || pad->pressed.R) {
    oslPlaySound(_switch, 2);
    selectNextItem();
  }
  if (pad->pressed.up || pad->pressed.L) {
    oslPlaySound(_switch, 2);
    selectPreviousItem();
  }
  if (pad->pressed.right) {
    if (_currentItem == 2) {_speedBar->addValue(1);}
    if (_currentItem == 3) {_levelBar->addValue(1);}
  }
  if (pad->pressed.left) {
    if (_currentItem == 2) {_speedBar->addValue(-1);}
    if (_currentItem == 3) {_levelBar->addValue(-1);}
  }
  if (pad->pressed.cross) {
    oslPlaySound(_validate, 1);
    if (_currentItem == 0) {
      startGame();
    }
    if (_currentItem == 1) {
      if (_mode < Game::GameModes - 1) {
        ++_mode;
      } else {
        _mode = 0;
      }
      setMode(_mode);
    }
    if (_currentItem == 4) {
      _fadeStatus = FadingOut;
      osl_quit = true;
    }
  }
  //Update items
  for (std::vector<Item*>::iterator it = _items.begin(); it != _items.end(); ++it) {
    (*it)->update();
  }
  //Update background animation
  _em.update();
}

void Menu::render()
{
  oslStartDrawing();
  oslClearScreen(RGB(0, 0, 0));
  //Render background
  for (std::vector<Particle*>::iterator it = _em._particules.begin(); it != _em._particules.end(); ++it) {
    drawImageFrameXY(_tile, (*it)->type, (*it)->pos.x, (*it)->pos.y);
  }
  //Menu space
  oslDrawFillRect(0, 0, 140, 272 , 0x8f000000);
  //Draw cursor
  oslDrawFillRect(MenuOffsetX - 8, MenuOffsetY + MenuItemSpacingY * _currentItem + 2, MenuOffsetX - 8 + CursorSize, MenuOffsetY + MenuItemSpacingY * _currentItem + 2 + CursorSize, 0x5fffffff);
  //Render items
  for (std::vector<Item*>::iterator it = _items.begin(); it != _items.end(); ++it) {
    (*it)->render();
  }
  //Draw fade mask
  oslDrawFillRect(0, 0, 480, 272 , _fadeAlpha << 24);
  oslEndDrawing();
}

void Menu::startGame()
{
  _game.newGame(_mode, _speedBar->getValue(), _levelBar->getValue());
  int skip = 0;
  OSL_CONTROLLER* pad;
  while (!osl_quit && !_game.isEnded()) {
    if (!skip) {
      pad = oslReadKeys();
      _game.update(pad);
      _game.render();
    }
    oslEndFrame();
    skip = oslSyncFrame();
  }
}

void Menu::showIntro()
{
  OSL_IMAGE* intro = oslLoadImageFilePNG((char*)"media/intro.png", OSL_IN_RAM, OSL_PF_8888);
  int skip = 0;
  int t = 0;
  int a = 0xff;
  while (t < 192) {
    if (!skip) {
      //Fade
      if (t < 128) {
        a -= 4;
        if (a < 0) {
          a = 0;
        }
      }
      if (t > 128) {
        a += 4;
        if (a > 255) {
          a = 255;
        }
      }
      //Draw
      oslStartDrawing();
      oslDrawImage(intro);
      oslDrawFillRect(0, 0, 480, 272 , a << 24);
      oslEndDrawing();
      ++t;
    }
    oslEndFrame();
    skip = oslSyncFrame();
  }
  oslDeleteImage(intro);
}

void Menu::selectNextItem()
{
  _items[_currentItem]->setFocus(false);
  if (_currentItem < MenuEntries - 1) {
    ++_currentItem;
  } else {
    _currentItem = 0;
  }
  _items[_currentItem]->setFocus(true);
  if (!_items[_currentItem]->isEnabled()) {
    selectNextItem();
  }
}

void Menu::selectPreviousItem()
{
  _items[_currentItem]->setFocus(false);
  if (_currentItem > 0) {
    --_currentItem;
  } else {
    _currentItem = MenuEntries - 1;
  }
  _items[_currentItem]->setFocus(true);
  if (!_items[_currentItem]->isEnabled()) {
    selectPreviousItem();
  }
}

void Menu::setMode(int mode)
{
  _mode = mode;
  if (_mode == Classic) {
    _modeText->setText("Classic");
    _items[3]->disable();
  }
  if (_mode == Garbage) {
    _modeText->setText("Garbage");
    _items[3]->enable();
  }
}
